{% extends "main.html" %}

{% block content %}
<div align="center">
  <h1>404 - Not found</h1>
  <figure>
    <img src="https://media.giphy.com/media/SmVVp30ymr3OznAOVj/giphy.gif" alt="GIF" width="300">
    <figcaption>
      <a href="https://giphy.com/doodles" target="_blank">@doodles</a>
    </figcaption>
  </figure>
  <p>there's nothing to see here!</p>
  <a href="/">Go Home.</a>
</div>
{% endblock %}
